# the installation for the SilvaSampleContentTypes product

from Products.Silva.install import add_fss_directory_view

def install(root):
    """install the view infrastructure for SilvaSampleContentTypes.
    """
    # create the core views from filesystem
    add_fss_directory_view(root.service_views, 
    'SilvaSampleContentTypes', __file__, 'views')

    # also register views
    registerViews(root.service_view_registry)
    # no widgets yet
    registerWidgets(root)

def uninstall(root):
    """uninstall the view infrastructure for SilvaSampleContentTypes.
    """
    unregisterWidgets(root)
    unregisterViews(root.service_view_registry)
    root.service_views.manage_delObjects(['SilvaSampleContentTypes'])

def is_installed(root):
    """test if SilvaSampleContentTypes are installed.
    """
    return hasattr(root.service_views, 'SilvaSampleContentTypes')

def registerViews(registry):
    """ register the views for all content types
    """
    registry.register('add', 'Silva Sample Blob', 
                      ('add', 'SampleContentTypes', 'Blob'))
    registry.register('edit', 'Silva Sample Blob', 
                      ('edit', 'VersionedContent', 'SampleContentTypes', 'Blob'))
    registry.register('public', 'Silva Sample Blob',
                      ('public', 'SampleContentTypes', 'Blob') )

    registry.register('add', 'Silva Sample Document', 
                      ('add', 'SampleContentTypes', 'SampleDocument'))
    registry.register('edit', 'Silva Sample Document', 
                      ('edit', 'VersionedContent', 'SampleContentTypes', 'SampleDocument'))
    registry.register('public', 'Silva Sample Document',
                      ('public', 'SampleContentTypes', 'SampleDocument') )

    registry.register('add', 'Silva Sample Meeting Agenda', 
                      ('add', 'SampleContentTypes', 'Agenda'))
    registry.register('edit', 'Silva Sample Meeting Agenda', 
                      ('edit', 'VersionedContent', 'SampleContentTypes', 'Agenda'))
    registry.register('public', 'Silva Sample Meeting Agenda',
                      ('public', 'SampleContentTypes', 'Agenda') )




def unregisterViews(registry):
    """ unregister the views for all content types
    """
    registry.unregister('add', 'Silva Sample Blob')
    registry.unregister('edit', 'Silva Sample Blob')
    registry.unregister('public', 'Silva Sample Blob')

    registry.unregister('add', 'Silva Sample Document')
    registry.unregister('edit', 'Silva Sample Document')
    registry.unregister('public', 'Silva Sample Document')

    registry.unregister('add', 'Silva Sample Meeting Agenda')
    registry.unregister('edit', 'Silva Sample Meeting Agenda')
    registry.unregister('public', 'Silva Sample Meeting Agenda')


def registerWidgets(root):
    """ register all XML widgets
    """

    # create the widgets directory view
    add_fss_directory_view(root, 
                           'service_sample_widgets', __file__, 'widgets')

    # tell the service editor to mix in this widget directory 
    root.service_editor.addWidgetRoot('service_sample_widgets', depends_on=('Silva',) )
    
    registerSampleDocWidgets(root)
    registerAgendaWidgets(root)


def unregisterWidgets(root):
    """ unregister all XML widgets
    """
    unregisterAgendaWidgets(root)
    unregisterSampleDocWidgets(root)
    root.service_editor.removeWidgetRoot('service_sample_widgets')
    root.manage_delObjects(['service_sample_widgets'])



def registerSampleDocWidgets(root):
    """ register all widgets needed for the Silva Sample Document
    """

    # create widget registries by copying from the document
    # XXX the previewer is not used yet ...
    root.manage_clone(root.service_doc_viewer, 'service_sample_document_viewer')
    root.manage_clone(root.service_doc_editor, 'service_sample_document_editor')

    doc_viewer = root.service_sample_document_viewer
    doc_editor = root.service_sample_document_editor

    doc_editor.addWidget('doc', ( 'top', 'samples', 'doc', 'mode_normal'))
    doc_viewer.addWidget('doc', ( 'top', 'samples', 'doc', 'mode_view'))

    # add special new elements
    for nested_item in ('disptitle', 'file'):
        doc_editor.addWidget(nested_item, ( 'element',  'samples', 'doc_elements', nested_item, 'mode_normal'))
        doc_viewer.addWidget(nested_item, ( 'element',  'samples', 'doc_elements', nested_item, 'mode_view'))
    
    
    doc_editor.setDisplayName('disptitle', 'Headline')
    doc_editor.setDisplayName('file', 'Download')

    allowed = list(doc_editor.getAllowed('doc'))
    allowed.append('file')
    doc_editor.setAllowed('doc', allowed )


def registerAgendaWidgets(root):
    """ register all widgets needed for the Silva Sample Meeting Agenda
    """

    # create widget registries for outer sequential nesting
    for name in ('service_sample_agenda_viewer','service_sample_agenda_editor'):
        root.manage_addProduct['XMLWidgets'].manage_addWidgetRegistry(name)

    agenda_viewer = root.service_sample_agenda_viewer
    agenda_editor = root.service_sample_agenda_editor

    # register top level node
    agenda_viewer.addWidget('agenda', ('top','samples','agenda','mode_view'))
    agenda_editor.addWidget('agenda', ('top','samples','agenda','mode_normal'))

    # register the only element node
    agenda_viewer.addWidget('talk',
                                 ('element', 'samples', 'agenda_elements', 'talk','mode_view'))
    agenda_editor.addWidget('talk',
                                 ('element', 'samples', 'agenda_elements','talk','mode_normal'))

    # tell the element names allowed for insert
    agenda_editor.setAllowed('agenda', ('talk',) )

    agenda_editor.setDisplayName('agenda', 'Agenda')
    agenda_editor.setDisplayName('talk', 'One Talk')
    


    # create widget registries for the inner amorphous mapping
    for name in ('service_sample_talk_viewer','service_sample_talk_editor'):
        root.manage_addProduct['XMLWidgets'].manage_addWidgetRegistry(name)
    
    talk_viewer = root.service_sample_talk_viewer
    talk_editor = root.service_sample_talk_editor

    # register the top level element
    talk_viewer.addWidget('talk',
                               ('fixed_size_top', 'samples', 'talk', 'generic', 'mode_view'))
    talk_editor.addWidget('talk',
                               ('fixed_size_top', 'samples', 'talk', 'generic', 'mode_edit'))

    # register child elements without corresponding formulator field
    # this is only the abstract yet

    talk_viewer.addWidget('abstract',
                               ('fixed_size_elements', 'samples', 'talk_elements', 'abstract', 'mode_view'))
    talk_editor.addWidget('abstract',
                               ('fixed_size_elements', 'samples', 'talk_elements', 'abstract', 'mode_edit'))

    # no need to call setAllowed, as no insert possible
    # display names only needed for non-formulator elements
    talk_editor.setDisplayName('talk', 'One Talk')
    talk_editor.setDisplayName('abstract','Abstract')


    # the next nesting where 'abstract' is the parent is actually
    # handles by the already exisiting service_sub_XXX registries (yet)
    # just register the top level node here (actually some kind of monkey patch ...)
    sub_viewer = root.service_sub_viewer
    sub_viewer.addWidget('abstract',
                         ('service_widgets', 'top', 'sub', 'mode_view'))
    sub_editor = root.service_sub_editor
    sub_editor.addWidget('abstract',
                         ('service_widgets', 'top', 'sub', 'mode_normal'))

    # as actually these elements may be transfored into each other,
    # register them all.
    sub_editor.setAllowed('abstract', ['p', 'heading', 'list', 'pre', 'dlist'])
    

def unregisterAgendaWidgets(root):
    """ unregister all widgets needed for the Silva Sample Meeting Agenda
    """

    root.manage_delObjects(['service_sample_agenda_viewer',
                            'service_sample_agenda_editor',
                            'service_sample_talk_viewer',
                            'service_sample_talk_editor'])


def unregisterSampleDocWidgets(root):
    """ unregister all widgets needed for the Silva Sample Document
    """

    to_delete = []
    for name in ('service_sample_document_editor','service_sample_document_viewer'):
        if hasattr(root.aq_explicit, name):
            to_delete.append(name)

    root.manage_delObjects(to_delete)
    
